<?php declare(strict_types=1);
/**
 * Banker
 *
 * A Caching library implementing psr/cache
 *
 * PHP version 7.0
 *
 * @package     Banker
 * @author      Timothy J. Warren <tim@timshomepage.net>
 * @copyright   2016  Timothy J. Warren
 * @license     http://www.opensource.org/licenses/mit-license.html  MIT License
 * @version     1.0.0
 * @link        https://git.timshomepage.net/timw4mail/banker
 */

namespace Aviat\Banker\Driver;

use Aviat\Banker\Exception\CacheException;

use Memcached;
use MemcachedException;

/**
 * Memcached cache backend
 */
class MemcachedDriver extends AbstractDriver {

	/**
	 * Driver for PHP Memcache extension
	 *
	 * @param array $config
	 * @param array $options
	 * @throws CacheException
	 */
	public function __construct(array $config = [], array $options = [])
	{
		// @codeCoverageIgnoreStart
		if ( ! class_exists('Memcached'))
		{
			throw new CacheException("Memcached driver requires memcached extensions");
		}
		// @codeCoverageIgnoreEnd

		try
		{
			$this->conn = new Memcached();
			$this->conn->setOption(Memcached::OPT_BINARY_PROTOCOL, true);
			$this->conn->addServer($config['host'], (int) $config['port']);

			// @codeCoverageIgnoreStart
			if ( ! empty($options))
			{
				$this->conn->setOptions($options);
			}
		}
		catch (MemcachedException $e)
		{
			// Rewrite MemcachedException as a CacheException to
			// match the requirements of the interface
			throw new CacheException($e->getMessage(), $e->getCode(), $e);
		}
		// @codeCoverageIgnoreEnd
	}

	/**
	 * Disconnect from memcached server
	 */
	public function __destruct()
	{
		$this->conn->quit();
	}

	/**
	 * See if a key currently exists in the cache
	 *
	 * @param string $key
	 * @return bool
	 */
	public function exists(string $key): bool
	{
		return $this->conn->get($key) !== FALSE;
	}

	/**
	 * Get the value for the selected cache key
	 *
	 * @param string $key
	 * @return mixed
	 */
	public function get(string $key)
	{
		return $this->conn->get($key);
	}

	/**
	 * Retrieve a set of values by their cache key
	 *
	 * @param string[] $keys
	 * @return array
	 */
	public function getMultiple(array $keys = []): array
	{
		return $this->conn->getMulti($keys);
	}

	/**
	 * Set a cached value
	 *
	 * @param string $key
	 * @param mixed $value
	 * @param int $expires
	 * @return DriverInterface
	 */
	public function set(string $key, $value, int $expires = 0): DriverInterface
	{
		if ( ! $this->exists($key))
		{
			$this->conn->add($key, $value, $expires);
		}
		else
		{
			$this->conn->replace($key, $value, $expires);
		}

		return $this;
	}

	/**
	 * Remove an item from the cache
	 *
	 * @param string $key
	 * @return boolean
	 */
	public function delete(string $key): bool
	{
		return $this->conn->delete($key);
	}

	/**
	 * Remove multiple items from the cache
	 *
	 * @param string[] $keys
	 * @return boolean
	 */
	public function deleteMultiple(array $keys = []): bool
	{
		return $this->conn->deleteMulti($keys);
	}

	/**
	 * Empty the cache
	 *
	 * @return boolean
	 */
	public function flush(): bool
	{
		return $this->conn->flush();
	}

	/**
	 * Set the specified key to expire at the given time
	 *
	 * @param string $key
	 * @param int $expires
	 * @return boolean
	 */
	public function expiresAt(string $key, int $expires): bool
	{
		if ($this->exists($key))
		{
			return $this->conn->touch($key, $expires);
		}

		$this->getLogger()->warn("Tried to set expiration on a key that does not exist");

		return FALSE;
	}
}
