<?php declare(strict_types=1);
/**
 * Banker
 *
 * A Caching library implementing psr/cache
 *
 * PHP version 7.0
 *
 * @package     Banker
 * @author      Timothy J. Warren <tim@timshomepage.net>
 * @copyright   2016  Timothy J. Warren
 * @license     http://www.opensource.org/licenses/mit-license.html  MIT License
 * @version     1.0.0
 * @link        https://git.timshomepage.net/timw4mail/banker
 */

namespace Aviat\Banker\Driver;


/**
 * Cache backend for use without a cache server. Only does transient
 * in-memory caching
 */
class NullDriver extends AbstractDriver {

	/**
	 * In memory store
	 *
	 * @var array
	 */
	protected $store = [];

	/**
	 * NullDriver constructor.
	 *
	 * @param array $config
	 * @param array $options
	 */
	public function __construct(array $config = [], array $options = [])
	{
		$this->store = [];
	}

	/**
	 * Clean up nothing
	 */
	public function __destruct()
	{
		//noop
	}

	/**
	 * See if a key currently exists in the cache
	 *
	 * @param string $key
	 * @return bool
	 */
	public function exists(string $key): bool
	{
		return array_key_exists($key, $this->store);
	}

	/**
	 * Get the value for the selected cache key
	 *
	 * @param string $key
	 * @return mixed
	 */
	public function get(string $key)
	{
		return ($this->exists($key))
			? $this->store[$key]
			: NULL;
	}

	/**
	 * Retrieve a set of values by their cache key
	 *
	 * @param string[] $keys
	 * @return array
	 */
	public function getMultiple(array $keys = []): array
	{
		$output = [];

		foreach($keys as $key)
		{
			$output[$key] = $this->get($key);
		}

		return $output;
	}

	/**
	 * Set a cached value
	 *
	 * @param string $key
	 * @param mixed $value
	 * @param int $expires
	 * @return DriverInterface
	 */
	public function set(string $key, $value, int $expires = 0): DriverInterface
	{
		$this->store[$key] = $value;
		return $this;
	}

	/**
	 * Remove an item from the cache
	 *
	 * @param string $key
	 * @return boolean
	 */
	public function delete(string $key): bool
	{
		unset($this->store[$key]);
		return ( ! array_key_exists($key, $this->store));
	}

	/**
	 * Remove multiple items from the cache
	 *
	 * @param string[] $keys
	 * @return boolean
	 */
	public function deleteMultiple(array $keys = []): bool
	{
		$res = TRUE;

		foreach($keys as $key)
		{
			$res = $res && $this->delete($key);
		}

		return $res;
	}

	/**
	 * Empty the cache
	 *
	 * @return boolean
	 */
	public function flush(): bool
	{
		$this->store = [];
		return TRUE;
	}

	/**
	 * Set the specified key to expire at the given time
	 *
	 * @param string $key
	 * @param int $expires
	 * @return boolean
	 */
	public function expiresAt(string $key, int $expires): bool
	{
		//noop
		return TRUE;
	}
}